#!/bin/bash
source globalvar.sh

main(){
    if [ $# -ne 5 ]; then  
        printUsage
        exit 1
    fi

    local inrequest=${1}
    local tag=${2}
    local maxbasenum=${3}
    local minduration=${4}
    local lpost=${5}

    # (0) build work directors
    buildDirectors
    
    # (1) 读取文件及生成配置文件
    echo "(1) VRS reading input and generating configuration ..."    
    python3 ${bindir}/py_getinput_form.py ${templatedir}/gnss.sit_out ${homedir}/tmp/outstation ${inrequest} ${homedir}/tmp/outrequest ${minduration}
    generateConfig

    # (2) 寻找最优三角网（有待优化）
    echo "(2) VRS Find the best triangle ..."
    multiFindTriangle $lpost
    
    # (3) 下载数据
    echo "(3) VRS downloading observation ..."
    if [ $lpost == "lpost" ];then
        ${bindir}/copy_allstation.sh ${homedir}/tmp/downstainform.txt ${homedir} #拷贝所有数据
    else
        ${bindir}/down_allstation.sh ${homedir}/tmp/downstainform.txt ${homedir}/tmp/outtype.txt  ${homedir} #下载所有数据并且进行拼接
    fi
     
    # (4) 生成基线配置信息config.txt 
    echo "(4) VRS generating config.txt ..."
    cat tmp/conf.txt > tmp/config.txt
    cat tmp/stationline.txt >> tmp/config.txt
    
    # (5) 更新所选测站的坐标
    echo "(5) Update the coordinate of the base station ..."
    prepareUpdateGnssSit
    ${bbobindir}/bamboo -mode rtk nrtk.json -post -itrs -dia -iar -repslip -pthread -pams -child -outmissar
    # pos_file to gnss.sit_out, then to xyzfile.txt
    updateGnssSit

    # # (6) 伪距平滑
    # echo "(6) smooth pseudo-range ..."
    # prepareSmoothRange
    # ${bbobindir}/obs_entry -mode ppp obs_entry.json -type prepro -obssmooth -post

    # (6) 解算NRTK，生成反向约束大气
    echo "(6) execute bamboo nrtk, generate atom by backward smooth ..."
    prepareBambooNrtk
    ${bindir}/pro_smooth.sh ./list    

    # (7) 生成虚拟观测值
    echo "(7) generate vrs obs ..."
    prepareBambooVrs
    ${bbobindir}/bamboo -mode vrs nrtk.json  -post -debug -vinput vrs_config.json

    # (8)生成报告
    echo "(8) VRS${num_request} Generate report ..."
    generateReport
 
    echo "All Finish!"  
}

# functions
function buildDirectors(){
    homedir=${workdir}/${tag}
    if [ -d ${homedir} ];then
        echo "${homedir} exists, so remove it" 
        rm -rf ${homedir}
    fi 
    mkdir -p ${homedir}
    mkdir -p ${homedir}/delaunary   # delaunary directory 
    mkdir -p ${homedir}/obs         # obs directory
    mkdir -p ${homedir}/tmp
    mkdir -p ${homedir}/tmp/result  # 存储每个请求三角形的结果，以便于后期合并
    mkdir -p ${homedir}/tmp_test    # 临时文件夹，用于存储每个单独请求对应的文件
    mkdir -p ${homedir}/tmp_test1   # 临时文件夹，用于存储和处理所有请求的原始文件
    obsdir_p=${homedir}/obs
    
    ## prepare basic configuration file and software
    cp -rf ${templatedir}/rt_nrtk_bak ${homedir}/rt_nrtk
    #cp -rf ${templatedir}/rt_prepro_smobs_bak ${homedir}/rt_prepro_smobs
    cp -rf ${templatedir}/rt_vrs_bak ${homedir}/rt_vrs
    cp -rf ${templatedir}/rt_update_pos_bak ${homedir}/rt_update_pos
    chmod 755 ${homedir}/
}

function correctAtom(){
    if [ -d ${homedir}/rt_nrtk/results/ ];then
        cd ${homedir}/rt_nrtk/results/
        for i in `ls ionambcon_???????_????-????`
        do
            bl=${i:9}
            missarfile="missarlog"${bl}
            if [ -s ${missarfile} ];then
                python3 ${bindir}/py_qcForAtom.py 1 1 ${i} ${missarfile}
                if [ -s "${i}_qc" ];then
                    mv ${i} ${i}_raw
                    mv ${i}_qc ${i}
                fi
            fi
        done
        for i in `ls ztdambcon_???????_????-????`
        do
            bl=${i:9}
            missarfile="missarlog"${bl}
            if [ -s ${missarfile} ];then
                python3 ${bindir}/py_qcForAtom.py 1 1 ${i} ${missarfile}
                if [ -s "${i}_qc" ];then
                    mv ${i} ${i}_raw
                    mv ${i}_qc ${i}
                fi
            fi
        done
    fi
}

function multiFindTriangle(){
    lpost=$1
    # 从 tmp 中引入原始测站列表和用户请求文件至临时文件夹 tmp_test1
    if [ -f ${homedir}/tmp/outstation ];then
        mv ${homedir}/tmp/outstation ${homedir}/tmp_test1/outstation
    fi
    if [ -f ${homedir}/tmp/outrequest ];then
        mv ${homedir}/tmp/outrequest ${homedir}/tmp_test1/outrequest
    fi

    # generateConfig() 会在 tmp_test 生成 outstation (tmp/ 中也有) 和 triangle_history.txt，其中前者需要删除，后者需要移到新建的临时文件夹 tmp_test1
    if [ -f ${homedir}/tmp_test/outstation ];then
        rm -f ${homedir}/tmp_test/outstation
    fi
    if [ -f ${homedir}/tmp_test/triangle_history.txt ];then
        mv ${homedir}/tmp_test/triangle_history.txt ${homedir}/tmp_test1/triangle_history.txt
    fi

    # 拆分 outrequest，以为每个请求生成独立的 outrequest 文件，并存储在 tmp_test1 中
    outrequest_rows=0
    if [ ! -f ${homedir}/tmp_test1/outrequest ];then
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The file is not exist ${homedir}/tmp_test1/outrequest"
        exit 1
    fi
    ## 获取用户请求的个数 outrequest_rows
    outrequest_rows=$(awk '{print NR}' ${homedir}/tmp_test1/outrequest | tail -n 1)
    let outrequest_rows=outrequest_rows-2 # outrequest 前两行为统一的文本内容，不应纳入计数
    if [ ${outrequest_rows} -lt 1 ];then
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The number of user request is less than 1"
        exit 1
    fi

    local i=0
    for ((i=1;i<=$outrequest_rows;i++));do
        if [ ! -f ${homedir}/tmp_test1/"outrequest$i" ];then
            touch ${homedir}/tmp_test1/"outrequest$i"
        fi
    done
    file_name="outrequest"
    row_num=0
    if [ -f ${homedir}/tmp_test1/outrequest ];then
        while read line
        do
            let row_num=row_num+1
            if [ $row_num -lt 3 ];then
                for ((i=1;i<=$outrequest_rows;i++));do
                    echo ${line} >> "${homedir}/tmp_test1/${file_name}${i}"
                done
            fi
            if [ $row_num -ge 2 ];then
                for ((i=1;i<=$outrequest_rows;i++));do
                    row_num1=0
                    let row_num1=row_num-2
                    if [ $row_num1 -eq $i ];then
                        echo ${line} >> "${homedir}/tmp_test1/${file_name}${i}"
                    fi
                done
            fi
        done < ${homedir}/tmp_test1/outrequest
    else
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The file is not exist ${homedir}/tmp_test1/outrequest"
        exit 1
    fi

    # 针对上一步生成的各个请求的独立请求文件，依次寻找三角形
    for ((i=1;i<=$outrequest_rows;i++));do
        # 准备输入文件
        if [ ! -f ${homedir}/tmp/outstation ];then
            cp ${homedir}/tmp_test1/outstation ${homedir}/tmp/outstation
        fi
        if [ ! -f ${homedir}/tmp/outrequest${i} ];then
            cp ${homedir}/tmp_test1/outrequest${i} ${homedir}/tmp/outrequest${i}
        fi
        if [ -f ${homedir}/tmp/outrequest${i} ];then
            mv ${homedir}/tmp/outrequest${i} ${homedir}/tmp/outrequest
        fi
        if [ ! -f ${homedir}/tmp_test/outstation ];then
            cp ${homedir}/tmp_test1/outstation ${homedir}/tmp_test/outstation
        fi
        if [ ! -f ${homedir}/tmp_test/triangle_history.txt ];then
            cp ${homedir}/tmp_test1/triangle_history.txt ${homedir}/tmp_test/triangle_history.txt
        fi

        # 调用函数，寻找三角形
        findTriangle $lpost

        # 整理结果
        if [ -f ${homedir}/tmp/downstainform.txt ];then
            mv ${homedir}/tmp/downstainform.txt ${homedir}/tmp/result/downstainform${i}.txt
        fi
        if [ -f ${homedir}/tmp/newusebasestation.txt ];then
            mv ${homedir}/tmp/newusebasestation.txt ${homedir}/tmp/result/newusebasestation${i}.txt
        fi
        if [ -f ${homedir}/tmp/stationline.txt ];then
            mv ${homedir}/tmp/stationline.txt ${homedir}/tmp/result/stationline${i}.txt
        fi
        if [ -f ${homedir}/tmp/xyzfile.txt ];then
            mv ${homedir}/tmp/xyzfile.txt ${homedir}/tmp/result/xyzfile${i}.txt
        fi
        if [ -f ${homedir}/tmp/conf.txt ];then
            mv ${homedir}/tmp/conf.txt ${homedir}/tmp/result/conf${i}.txt
        fi
        if [ -f ${homedir}/tmp/alterLog.txt ];then
            mv ${homedir}/tmp/alterLog.txt ${homedir}/tmp/result/alterLog${i}.txt
        fi
        
        # 清理tmp和tmp_test中的文件
        # 因为: 1)后续请求需要重新去 tmp_test1 中拉去原始未改动的信息文件
        #       2)后续 tmp/outrequest 和 tmp/outstation 要根据 tmp/result 中的结果合并生成
        rm ${homedir}/tmp_test/*
        rm ${homedir}/tmp/*.txt
        rm ${homedir}/tmp/outrequest
        rm ${homedir}/tmp/outstation
    done

    # 合并单个请求生成的结果
    # /tmp/result 中包含所有请求生成的结果:
    #     1) conf-id-.txt              记录了每个请求的起止时间和要用到的观测值/广播星历的路径
    #     2) downstainform-id-.txt     记录了每个请求需要的三个基站的下载信息
    #     3) newusebasestation-id-.txt 记录了每个请求的起止时间和距离最近的基站信息
    #     4) stationline-id-.txt       记录了每个请求需要的三条基线以及基线数目
    #     5) xyzfile-id-.txt           记录了每个请求需要的三个基站的坐标
    # 最终需要合并生成
    #     a) config.txt                由 1) 和 4) 组成
    #     b) downstainform.txt         由 2) 组成
    #     c) newusebasestation.txt     由 3) 组成
    #     d) xyzfile.txt               由 5) 组成
    # 注意:
    #     由于采用新的 py_getinput_form.py，现在其中包含的起止时间和跨度都得到了统一，所以可以不用考虑合并时间信息
    #     比如 1) 2) 3) 中的时间信息
    if [ ! -d ${homedir}/tmp/result ];then
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The directory with triangle results is not exist ${homedir}/tmp/result"
        exit 1
    fi
    ## prepare the empty result file
    if [ -f ${homedir}/tmp/config.txt ];then
        rm -f ${homedir}/tmp/config.txt
    fi
    if [ -f ${homedir}/tmp/conf.txt ];then
        rm -f ${homedir}/tmp/conf.txt
    fi
    if [ -f ${homedir}/tmp/stationline.txt ];then
        rm -f ${homedir}/tmp/stationline.txt
    fi
    if [ -f ${homedir}/tmp/downstainform.txt ];then
        rm -f ${homedir}/tmp/downstainform.txt
    fi
    if [ -f ${homedir}/tmp/newusebasestation.txt ];then
        rm -f ${homedir}/tmp/newusebasestation.txt
    fi
    if [ -f ${homedir}/tmp/xyzfile.txt ];then
        rm -f ${homedir}/tmp/xyzfile.txt
    fi

    ## config.txt
    touch ${homedir}/tmp/config.txt
    touch ${homedir}/tmp/conf.txt
    touch ${homedir}/tmp/stationline.txt

    for t_conf in `ls ${homedir}/tmp/result/conf*.txt`;do
        cat ${t_conf} >> ${homedir}/tmp/conf.txt
    done
    ### 注意: 由于现在采用先统一请求的起止时间的方式，所有请求生成的 conf-id-.txt实际上是一样的，所以只需要在不改变顺序的前提下删除重复行即可
    awk '!seen[$0]++' ${homedir}/tmp/conf.txt > ${homedir}/tmp/conf.txt_tmp
    if [ -s ${homedir}/tmp/conf.txt_tmp ];then
        mv ${homedir}/tmp/conf.txt_tmp ${homedir}/tmp/conf.txt
    else
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The temporated conf.txt_tmp is not exist or is empty ${homedir}/tmp/conf.txt_tmp"
        exit 1
    fi

    for t_staline in `ls ${homedir}/tmp/result/stationline*.txt`;do
        cat ${t_staline} >> ${homedir}/tmp/stationline.txt
    done
    python3 ${bindir}/py_merge_stationline.py ${homedir}/tmp/stationline.txt
    if [ -s ${homedir}/tmp/stationline.txt_tmp ];then
        mv ${homedir}/tmp/stationline.txt_tmp ${homedir}/tmp/stationline.txt
    else
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The temporated stationline.txt_tmp is not exist or is empty ${homedir}/tmp/stationline.txt_tmp"
        exit 1
    fi

    cat ${homedir}/tmp/conf.txt >> ${homedir}/tmp/config.txt
    cat ${homedir}/tmp/stationline.txt >> ${homedir}/tmp/config.txt
    if [ ! -s ${homedir}/tmp/config.txt ];then
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The config.txt is not exist or is empty ${homedir}/tmp/config.txt"
        exit 1
    fi

    ## newusebasestation.txt
    touch ${homedir}/tmp/newusebasestation.txt
    for t_newuse in `ls ${homedir}/tmp/result/newusebasestation*.txt`;do
        cat ${t_newuse} >> ${homedir}/tmp/newusebasestation.txt
    done
    sort -u ${homedir}/tmp/newusebasestation.txt > ${homedir}/tmp/newusebasestation.txt_tmp
    if [ -s ${homedir}/tmp/newusebasestation.txt_tmp ];then
        sed -i '/Grid                                     start_time          end_time            min_d_base/d' ${homedir}/tmp/newusebasestation.txt_tmp
        sed -i '1s/^/Grid                                     start_time          end_time            min_d_base\n/' ${homedir}/tmp/newusebasestation.txt_tmp

        mv ${homedir}/tmp/newusebasestation.txt_tmp ${homedir}/tmp/newusebasestation.txt
    else
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The temporated newusebasestation.txt_tmp is not exist or is empty ${homedir}/tmp/newusebasestation.txt_tmp"
        exit 1
    fi

    ## downstainform.txt
    touch ${homedir}/tmp/downstainform.txt
    for t_downsta in `ls ${homedir}/tmp/result/downstainform*.txt`;do
        cat ${t_downsta} >> ${homedir}/tmp/downstainform.txt
    done
    sort -u ${homedir}/tmp/downstainform.txt > ${homedir}/tmp/downstainform.txt_tmp
    if [ -s ${homedir}/tmp/downstainform.txt_tmp ];then
        mv ${homedir}/tmp/downstainform.txt_tmp ${homedir}/tmp/downstainform.txt
    else
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The temporated downstainform.txt_tmp is not exist or is empty ${homedir}/tmp/downstainform.txt_tmp"
        exit 1
    fi

    ## xyzfile.txt
    touch ${homedir}/tmp/xyzfile.txt
    for t_xyzfile in `ls ${homedir}/tmp/result/xyzfile*.txt`;do
        cat ${t_xyzfile} >> ${homedir}/tmp/xyzfile.txt
    done
    sort -u ${homedir}/tmp/xyzfile.txt > ${homedir}/tmp/xyzfile.txt_tmp
    if [ -s ${homedir}/tmp/xyzfile.txt_tmp ];then
        mv ${homedir}/tmp/xyzfile.txt_tmp ${homedir}/tmp/xyzfile.txt
    else
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The temporated xyzfile.txt_tmp is not exist or is empty ${homedir}/tmp/xyzfile.txt_tmp"
        exit 1
    fi

    ## check the result
    if [ ! -s ${homedir}/tmp/newusebasestation.txt ];then
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The newusebasestation.txt is not exist or is empty ${homedir}/tmp/newusebasestation.txt"
        exit 1
    fi
    if [ ! -s ${homedir}/tmp/downstainform.txt ];then
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The downstainform.txt is not exist or is empty ${homedir}/tmp/downstainform.txt"
        exit 1
    fi
    if [ ! -s ${homedir}/tmp/xyzfile.txt ];then
        echo "***ERROR(bbovrs_post.sh multifindTriangle()): The xyzfile.txt is not exist or is empty ${homedir}/tmp/xyzfile.txt"
        exit 1
    fi
}

function findTriangle(){
    local i=0
    lpost=$1
    find_num=-1 #表示寻找次数
    for ((i=50;i<=${maxbasenum};i++));do
        #delete not have date station(updateoutstation)
        let find_num=find_num+1
        tag_outstation=''
        if [ $find_num -ne 0 -a $find_num -lt 3 ];then
            tag_outstation="$find_num" #1 2 两种情况单独考虑
            if [  ! -f ${homedir}/tmp_test/outstation ];then
                cp ${homedir}/tmp/outstation ${homedir}/tmp_test/outstation #这两种情况需要更新
            fi
        fi
        if [  ! -f ${homedir}/tmp_test/outstation ];then
            cp ${homedir}/tmp/outstation ${homedir}/tmp_test/outstation 
        fi
        #echo outstation${tag_outstation}
        if [ -s ${homedir}/tmp_test/NODate.txt  -a $find_num -lt 3 ];then
            updateoutstation ${homedir}/tmp/outstation${tag_outstation} ${homedir}/tmp_test/NODate.txt #更新源头
        else
            updateoutstation ${homedir}/tmp_test/outstation${tag_outstation} ${homedir}/tmp_test/NODate.txt #更新最新
        fi


        python3 ${bindir}/py_findmindis_station.py ${homedir}/tmp_test/outstation${tag_outstation} ${homedir}/tmp/outrequest ${homedir}/tmp_test/usebasestation.txt  ${i}
        if [ ! -f ${homedir}/tmp_test/usebasestation.txt ];then
            echo "***ERROR(vrs_post.sh): the out file of py_findmindis_station.py is not exist: ${homedir}/tmp_test/usebasestation.txt"
            exit 1
        fi
        if [ $find_num -eq 0 ];then
            cp ${homedir}/tmp_test/usebasestation.txt ${homedir}/tmp/usebasestation.txt
        fi

        python3 ${bindir}/py_getobsdw_form.py ${homedir}/tmp_test/usebasestation.txt ${homedir}/tmp_test/downstainform.txt  ${i}
        if [ ! -f ${homedir}/tmp_test/downstainform.txt ];then
            echo "***ERROR(vrs_post.sh): the out file of py_getobsdw_form.py is not exist: ${homedir}/tmp_test/downstainform.txt"
            exit 1
        fi
        if [ $find_num -eq 0 ];then
            cp ${homedir}/tmp_test/downstainform.txt ${homedir}/tmp/downstainform.txt
        fi
        python3 ${bindir}/py_getoutput_form.py ${homedir}/tmp_test/downstainform.txt ${homedir}/tmp_test/outstation${tag_outstation} ${obsdir_p} ${brdmdir} ${homedir}/tmp_test/conf.txt ${homedir}/tmp_test/xyzfile.txt ${homedir}/tmp_test/xyzblhfile
        if [ $find_num -eq 0 ];then
            cp ${homedir}/tmp_test/xyzfile.txt ${homedir}/tmp/xyzfile.txt
        fi
        mv ${homedir}/tmp_test/xyzblhfile ${homedir}/delaunary
        cd ${homedir}/delaunary
        ${bindir}/Delaunary
        cd ${homedir}
        mv ${homedir}/delaunary/outtriangles.txt ${homedir}/tmp_test/
        #echo ${find_num}
        if [ ! -f ${homedir}/tmp_test/triangle ];then
            touch ${homedir}/tmp_test/triangle
        fi

        python3 ${bindir}/py_find_triangle.py ${homedir}/tmp/outrequest ${homedir}/tmp_test/xyzfile.txt ${homedir}/tmp_test/outtriangles.txt ${homedir}/tmp_test/stationline.txt ${homedir}/tmp_test/trianglestation.txt 115 25 ${homedir}/tmp_test/deletesta.txt ${find_num} ${homedir}/tmp_test/triangle
        #add 优化三角网(根据三角形内角进行优化)
        if [ -s ${homedir}/tmp_test/deletesta.txt ];then
            updateoutstation ${homedir}/tmp_test/outstation ${homedir}/tmp_test/deletesta.txt #只更新一个
        fi
        
        # if [ ${find_num} -eq 2 ];then
        #     exit 1
        # fi
        if [ -s ${homedir}/tmp_test/deletesta.txt ];then
            rm ${homedir}/tmp_test/deletesta.txt
        fi
        let num_file=find_num+1
        if [ $find_num -lt 3 ];then
            cp ${homedir}/tmp_test/outstation ${homedir}/tmp_test/outstation${num_file}
            rm ${homedir}/tmp_test/outstation
        fi
        #add 保存之前找到过的三角形
        cat ${homedir}/tmp_test/triangle >> ${homedir}/tmp_test/triangle_history.txt
        if [ -s ${homedir}/tmp_test/triangle ];then
            rm ${homedir}/tmp_test/triangle
        fi
        if [ ! -s ${homedir}/tmp_test/stationline.txt -a $i -eq ${maxbasenum} -a -s ${homedir}/tmp_test/triangle_history.txt ];then
            python3 ${bindir}/py_findhistory_triangle.py ${homedir}/tmp/outstation ${homedir}/tmp_test/triangle_history.txt ${homedir}/tmp_test/stationline.txt ${homedir}/tmp_test/trianglestation.txt
            #删除usebasestation.txt和downstationform.txt
            if [ -f ${homedir}/tmp_test/usebasestation.txt ];then
                rm ${homedir}/tmp_test/usebasestation.txt
            fi
            if [ -f ${homedir}/tmp_test/downstainform.txt ];then
                rm ${homedir}/tmp_test/downstainform.txt
            fi
            if [ -f ${homedir}/tmp_test/xyzfile.txt ];then
                rm ${homedir}/tmp_test/xyzfile.txt
            fi
            #用最全信息文件替换
            #因为后面画图会根据最近距离点进行，这里需要更新为最近点在前面
            mv ${homedir}/tmp/usebasestation.txt ${homedir}/tmp_test/usebasestation.txt  
            mv ${homedir}/tmp/downstainform.txt ${homedir}/tmp_test/downstainform.txt
            mv ${homedir}/tmp/xyzfile.txt ${homedir}/tmp_test/xyzfile.txt

            echo "Only suboptimal triangles can be selected!"
        fi

        if [ -s ${homedir}/tmp_test/stationline.txt ];then
            echo "Find a Triangle for VRS grid and ${num_file} selections made!"
        fi
        if [ ! -s ${homedir}/tmp_test/stationline.txt ];then
            echo "Not Find a Triangle for VRS grid and ${num_file} selections made!"
            if [ $i -eq ${maxbasenum} ];then
                echo "No suitable triangle found from maxbasenum and ${num_file} selections made!"
                exit 1
            fi
            continue
        fi
        #update downstainform.txt
        updatedwstaform ${homedir}/tmp_test/downstainform.txt ${homedir}/tmp_test/trianglestation.txt
        #upgrade xyzfile.txt(trianglestation.txt)
        updatexyzfile ${homedir}/tmp_test/trianglestation.txt ${homedir}/tmp_test/xyzfile.txt
        #更新usebasestation.txt中的最近点
        if [ -f ${homedir}/tmp_test/newusebasestation.txt ];then
            rm ${homedir}/tmp_test/newusebasestation.txt
        fi
        python3 ${bindir}/py_update_usebasestation.py ${homedir}/tmp_test/usebasestation.txt ${homedir}/tmp_test/xyzfile.txt ${homedir}/tmp_test/newusebasestation.txt
        #Judge whether the data exists
        if [ $lpost != "lpost" ];then
            ${bindir}/judge_obsexists.sh ${homedir}/tmp_test/downstainform.txt ${homedir}/tmp_test/outtype ${homedir}/tmp_test
        fi

        if [ -f ${homedir}/tmp_test/downstainform.txt ];then
            mv ${homedir}/tmp_test/*.txt ${homedir}/tmp/
            #rm -r ${homedir}/tmp_test/
            echo "Successfully obtained triangle!"
            break
        fi
        if [ ! -f ${homedir}/tmp_test/downstainform.txt ];then
            #rm -r ${homedir}/tmp_test/*
            echo "No suitable triangle found!"
        fi
    done
}

function generateConfig(){
    if [ ! -f ${homedir}/tmp/outstation ];then
        echo "***ERROR(vrs_post.sh): the output file of py_getinput_form.py is not exist: ${homedir}/tmp/outstation"
        exit 1
    fi
    if [ ! -f ${homedir}/tmp/outrequest ];then
        echo "***ERROR(vrs_post.sh): the output file of py_getinput_form.py is not exist: ${homedir}/tmp/outrequest"
        exit 1
    fi
    cp ${homedir}/tmp/outstation ${homedir}/tmp_test/outstation
    touch ${homedir}/tmp_test/triangle_history.txt #保存历史三角形
    ## b) parse request and rewrite the data.json
    parseRequest ${homedir}/tmp/outrequest
    ## Tip: need to check whether rewrite data.json successfully or not
}

function generateReport(){
    freqs=${homedir}/tmp_test1/outrequest # 总的 outrequest 已经移至 tmp_test1, 而非 tmp
    fconfig=${homedir}/tmp/config.txt
    cd ${homedir}/rt_vrs/results/
    local syear=$(awk '{if($1~/^TIME/) printf("%d", $2)}' ${fconfig})
    local smonth=$(awk '{if($1~/^TIME/) printf("%d", $3)}' ${fconfig})
    local sday=$(awk '{if($1~/^TIME/) printf("%d", $4)}' ${fconfig})
    local sh=$(awk '{if($1~/^TIME/) printf("%d", $5)}' ${fconfig})
    local sm=$(awk '{if($1~/^TIME/) printf("%d", $6)}' ${fconfig})
    local ss=$(awk '{if($1~/^TIME/) printf("%d", $7)}' ${fconfig})
    local seslen=$(awk '{if($1~/^TIME/) printf("%d", $8)}' ${fconfig})
    local mjd=$(ymd2mjd $syear $smonth $sday)
    local YDOY=($(mjd2ydoy $mjd))
    local doy=${YDOY[1]}
    local yy=${syear:2:2}
    local stime=$(date +%s -d "${syear}-${smonth}-${sday} ${sh}:${sm}:${ss}")
    stime=$((${stime}+1800))
    stime_new=$(date +%Y-%m-%d_%H:%M:%S -d "1970-01-01 UTC ${stime} seconds");
    seslen=$(($seslen - 1800))

    if [ ! -f $freqs ];then
        echo "can't open $freqs"
        exit
    fi
    local num_row=0
    while read line
    do
        let num_row=num_row+1
        if [ ${num_row} -lt 3 ];then
            continue
        fi
        echo $line
        local sitn=`echo $line | awk -F " " '{print $16}'`
        echo $sitn
        local sitn_low=$(echo $sitn | tr  '[A-Z]' '[a-z]')
        if [ ! -f ${sitn_low}${doy}0.${yy}o ];then
            echo "can't open ${sitn_low}${doy}0.${yy}o"
            continue
        fi
        #${bindir}/gfzrnx_lx -finp ${sitn_low}${doy}0.${yy}o -fout ${sitn_low}${doy}0.${yy}o_gfz0 -kv
        cp -f ${sitn_low}${doy}0.${yy}o temp${doy}0.${yy}o_gfz
        ${bindir}/gfzrnx_lx -finp temp${doy}0.${yy}o_gfz -kv > temp${doy}0.${yy}o_gfz_ok
        if [ -f "temp${doy}0.${yy}o_gfz_ok" ];then
            ${bindir}/anubis-3.4-lin-static-64b :inp:rinexo temp${doy}0.${yy}o_gfz_ok :inp:rinexn ${brdmdir}/brdm${doy}0.${yy}p :out:xtr ${sitn_low}${doy}0.xtr :out:log ${sitn_low}${doy}0.log :qc:sec_sum=9
            rm -f temp${doy}0.${yy}o_gfz_ok
        fi
        rm -f temp${doy}0.${yy}o_gfz

    done < $freqs
}

function parseRequest(){
    local reqf=$1
    local indata=$2
    local num_row=0   #记录行数
    local doy=0
    #写控制文件
    if [ -f $reqf ];then
        while read line
        do
            if [ ${num_row} -gt 1 ];then
                #转换为格网点名字VR01....   坐标换成XYZ
                grid_lon=`echo $line | awk -F " " '{print $1}'`     #获取变量
                grid_lat=`echo $line | awk -F " " '{print $2}'`     
                grid_height=`echo $line | awk -F " " '{print $3}'`
                grid_id=`echo $line | awk -F " " '{print $16}'`      #直接是名字
                syear=`echo $line | awk -F " " '{print $4}'`
                smonth=`echo $line | awk -F " " '{print $5}'`
                sday=`echo $line | awk -F " " '{print $6}'`
                shour=`echo $line | awk -F " " '{print $7}'`
                sminute=`echo $line | awk -F " " '{print $8}'`
                ssec=`echo $line | awk -F " " '{print $9}'`

                mjd=$(ymd2mjd $syear $smonth $sday)
                YDOY=($(mjd2ydoy $mjd))   
                doy=${YDOY[1]} 
                if [ $doy -lt 100 ];then
                    doy="0$doy"
                fi

                eyear=`echo $line | awk -F " " '{print $10}'`
                emonth=`echo $line | awk -F " " '{print $11}'`
                eday=`echo $line | awk -F " " '{print $12}'`
                ehour=`echo $line | awk -F " " '{print $13}'`
                eminute=`echo $line | awk -F " " '{print $14}'`
                esec=`echo $line | awk -F " " '{print $15}'`
                alltime=`echo $line | awk -F " " '{print $17}'`
                tmp=`blh2xyz $grid_lat $grid_lon $grid_height`  #获取字符串
                arr_XYZ=($tmp)                                  #字符串转数组
                #对格网点进行更新
                ## should look into data.json
		        ExecuteWithoutOutput "python3 ${bindir}/py_writereqs_form.py ${homedir}/rt_vrs/vrs_config.json reqs=${grid_id}+${arr_XYZ[0]}+${arr_XYZ[1]}+${arr_XYZ[2]}+${syear}+${smonth}+${sday}+${shour}+${sminute}+${ssec}+$alltime"
            fi
            let num_row=num_row+1
        done < $reqf
    fi
}

function prepareBambooNrtk(){
    cd ${homedir}/rt_nrtk
    ${bindir}/updateTableJson.py ./tables.json  ${tablesdir}
    prepareConfigBBO ${homedir}/tmp/xyzfile.txt ./nrtk.json
    python3 ${bindir}/py_getbaseline_form.py ${homedir}/tmp/stationline.txt ./list
}

function prepareBambooVrs(){
    cd ${homedir}/rt_vrs
    ${bindir}/updateTableJson.py ./tables.json  ${tablesdir}
    prepareConfigBBO ${homedir}/tmp/xyzfile.txt ./nrtk.json
    python3 ${bindir}/py_getbaseline_form.py ${homedir}/tmp/stationline.txt ./list
    cp ${homedir}/rt_nrtk/results/ionambcon_???????_????-???? ./external_input
    cp ${homedir}/rt_nrtk/results/ztdambcon_???????_????-???? ./external_input
    #cp ${homedir}/rt_prepro_smobs/results/????????.??o ./obs/
    #sed -i "s#\"obs\":\"${obsdir_p}/-YYYY-/-DDD-/\",#\"obs\":\"./obs\",#g" ./nrtk.json
}

function prepareConfigBBO(){
    local xyz=$1
    local workjson=$2
    cat $xyz | awk '{print $1}' > sitlist
    #${bindir}/makeSes.py -f session.obj_out -l sitlist -o session.obj_out_tmp -m F -s GECJ --dztd 0.1 --qztd 0.0001 --pgps 10:0.02 --pgal 10:0.02 --pglo 10:0.02 --pcmp 10:0.02 --pqzs 10:0.02
    ${bindir}/makeSes.py -f session_nrtk.obj -l sitlist -o session.obj_out -m F -s GECJ --dztd 0.1 --qztd .0004
    worktime=$(awk '{if($1~/^TIME/) printf("%d %d %d %d %d %d %d", $2,$3,$4,$5,$6,$7,$8)}' ${homedir}/tmp/config.txt)
    sed -i "s#\"time\":.*#\"time\":\"${worktime}\",#g" ${workjson}
    sed -i "s#\"broad\":.*#\"broad\":\"${brdmdir}/brdm-DDD-0.-YY-p\",#g" ${workjson}
    sed -i "s#\"obs\":.*#\"obs\":\"${obsdir_p}/-YYYY-/-DDD-/\",#g" ${workjson}
    ## make up coordinates 
    ${bindir}/makeSit.py -l sitlist -o gnss.sit_out 
    ${bindir}/pos2sit.py --fxyz $xyz -s gnss.sit_out 
    mv gnss.sit_out_tmp gnss.sit_out
}

function prepareSmoothRange(){
    cd ${homedir}/rt_prepro_smobs
    ${bindir}/updateTableJson.py ./tables.json  ${tablesdir}
    prepareConfigBBO ${homedir}/tmp/xyzfile.txt ./obs_entry.json
    python3 ${bindir}/py_getbaseline_form.py ${homedir}/tmp/stationline.txt ./list
}

function prepareUpdateGnssSit(){
    # update the xyzfile.txt
    if [ ! -d "${homedir}/rt_update_pos" ];then
        exit 1
    fi
    cd ${homedir}/rt_update_pos
    ${bindir}/updateTableJson.py ./tables.json  ${tablesdir}
    prepareConfigBBO ${homedir}/tmp/xyzfile.txt ./nrtk.json
    # we need build the ./list according to the xyzfile.txt, not the stationline.txt
    ## generate a temporate stationline.txt
    python3 ${bindir}/py_getStationLine_fromXyzFile.py ${homedir}/tmp/xyzfile.txt ./stationline_tmp.txt
    ## use the new stationline.txt to build the ./list
    python3 ${bindir}/py_getbaseline_form.py ./stationline_tmp.txt ./list
}

function printUsage(){
    echo "RTK_POST script usage:"
    echo "  bbovrs_post.sh reqs_input tag basenum minduration lpost"
    echo "Example:"
    echo "  bbovrs_post.sh request_sz1119 tag 60 600"
    echo "Input:"
    echo "  reqs_input  :   the require or setting of data processing, containing of the position of"
    echo "                  VRS. The format is defined by BaiDu"
    echo "  tag         :   the label of workspace, in order to avoid working directory conflicts"
    echo "  basenum     :   base station count that used to generate atomsphere"
    echo "  minduration :   minimum duration"
    echo "  lpost       :   execute using post mode"
}

function updateoutstation(){
    local outstationdir=$1
    local nodatedir=$2
    station1=""
    if [ -f $nodatedir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            station1=`echo $sitn_low | awk -F " " '{print $1}'`
            #echo $station1
            num=0
            if [ -f $outstationdir ];then
                while read line1
                do
                    sitn_low1=$(echo $line1 | tr  '[A-Z]' '[a-z]') 
                    station=`echo $sitn_low1 | awk -F " " '{print $1}'`
                    let num=num+1
                    if [ "$station1" == "$station" ];then
                        sed -i "${num}d" $outstationdir
                        let num=num-1
                    fi
                done < $outstationdir
            fi

        done < $nodatedir
    fi

}

function updatexyzfile(){
    local trianglestationdir=$1
    local xyzfiledir=$2
    local num=0
    local station1_id=0
    local station2_id=0
    local station3_id=0
    if [ -f $trianglestationdir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            station1=`echo $sitn_low | awk -F " " '{print $1}'`
            #station1_id=$((10#${station1:2:2}))
            station2=`echo $sitn_low | awk -F " " '{print $2}'`
            #station2_id=$((10#${station2:2:2}))
            station3=`echo $sitn_low | awk -F " " '{print $3}'`
            #station3_id=$((10#${station3:2:2}))
        done < $trianglestationdir
    fi
    if [ -f $xyzfiledir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            let num=num+1
            stationname=`echo $sitn_low | awk -F " " '{print $1}'`
            #station_id=$((10#${stationname:2:2}))
            if [ "$stationname" == "$station1" ];then
                continue
            fi
            if [ "$stationname" == "$station2"  ];then
                continue
            fi
            if [ "$stationname" == "$station3"  ];then
                continue
            fi
            sed -i "${num}d" $xyzfiledir
            let num=num-1
        done < $xyzfiledir
    fi

}

function updatedwstaform(){
    local downstainformdir=$1   #测站下载数据信息地址
    local trianglestationdir=$2   #基线信息
    local num=0
    local station1_id=0
    local station2_id=0
    local station3_id=0
    if [ -f $trianglestationdir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            station1=`echo $sitn_low | awk -F " " '{print $1}'`
            #station1_id=$((10#${station1:2:2}))
            station2=`echo $sitn_low | awk -F " " '{print $2}'`
            #station2_id=$((10#${station2:2:2}))
            station3=`echo $sitn_low | awk -F " " '{print $3}'`
            #station3_id=$((10#${station3:2:2}))
        done < $trianglestationdir
    fi
    if [ -f $downstainformdir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            let num=num+1
            stationname=`echo $sitn_low | awk -F " " '{print $13}'`
            #station_id=$((10#${stationname:2:2}))
            if [ "$stationname" == "$station1" ];then
                continue
            fi
            if [ "$stationname" == "$station2"  ];then
                continue
            fi
            if [ "$stationname" == "$station3"  ];then
                continue
            fi
            sed -i "${num}d" $downstainformdir
            let num=num-1
        done < $downstainformdir
    fi
}

# UpdateGnssSit: used for update the coordinate of site in the gnss.sit_out, and extract new xyzfile.txt from this gnss.sit_out
# Usage
function updateGnssSit() {
    if [ ! -d "${homedir}/rt_update_pos" ];then
        exit 1
    fi
    cd ${homedir}/rt_update_pos
    
    if [ ! -d "${homedir}/rt_update_pos/results" ];then
        exit 1
    fi

    posfile=`find ${homedir}/rt_update_pos/results -type f -name "pos_*" -print | head -n 1`
    if [ ! -f ${posfile} ];then
        exit 1
    fi
    ${bindir}/pos2sit.py -p ${posfile} -s gnss.sit_out
    mv gnss.sit_out_tmp gnss.sit_out

    if [ ! -f "${homedir}/rt_update_pos/gnss.sit_out" ];then
        exit 1
    fi
    python3 ${bindir}/py_getXyzFile_fromGnssSit.py ${homedir}/rt_update_pos/gnss.sit_out ${homedir}/rt_update_pos/xyzfile.txt

    if [ ! -f "${homedir}/rt_update_pos/xyzfile.txt" ];then
        exit 1
    fi
    if [ ! -f "${homedir}/tmp/xyzfile.txt" ];then
        exit 1
    else
        mv ${homedir}/tmp/xyzfile.txt ${homedir}/tmp/xyzfile.txt_old
        cp ${homedir}/rt_update_pos/xyzfile.txt ${homedir}/tmp/xyzfile.txt
    fi
}

# utils
function ExecuteWithoutOutput() {
    local cmd="$1"
    time=$(date +'%Y-%m-%d %H:%M:%S')
    $cmd >/dev/null 2>&1 
    if [ $? -eq 0 ]; then  
        #echo "脚本实现"
        return 0
    else
        echo "Execute ERROR!"
        return 1
    fi
}

function blh2xyz(){
    local a=6378137.0  
    local b=6356752.31414 
    local pi=3.141592653589793
    local B=$1
    local L=$2
    local H=$3
    B=$(echo $B $pi|awk '{printf "%0.8f\n", $1*$2/180}')
    L=$(echo $L $pi|awk '{printf "%0.8f\n", $1*$2/180}')
    e=$(echo $a $b|awk '{printf "%0.8f\n",sqrt(($1^2-$2^2)/$1^2)}')
    N=$(echo $a $e $B|awk '{printf "%0.8f\n",$1/(sqrt(1-$2^2*sin($3)*sin($3)))}')
    X=$(echo $N $H $B $L|awk '{printf "%0.8f\n",($1+$2)*cos($3)*cos($4)}')
    Y=$(echo $N $H $B $L|awk '{printf "%0.8f\n",($1+$2)*cos($3)*sin($4)}')
    Z=$(echo $N $H $B $e|awk '{printf "%0.8f\n",($1*(1-$4*$4)+$2)*sin($3)}')
    echo $X $Y $Z
}

function ymd2mjd() {
    local year=$1
    local mon=$((10#$2))
    local day=$((10#$3)) 
    [ $year -lt 100 ] && year=$((year + 2000))
    if [ $mon -le 2 ]; then
        mon=$(($mon + 12))
        year=$(($year - 1))
    fi
    local mjd=$(echo $year | awk '{print $1*365.25-$1*365.25%1-679006}')
    mjd=$(echo $mjd $year $mon $day | awk '{print $1+int(30.6001*($3+1))+2-int($2/100)+int($2/400)+$4}')
    echo $mjd
}

function mjd2ydoy() {
    local mjd=$1
    local year=$((($mjd + 678940) / 365))
    local mjd0=$(ymd2mjd $year 1 1)
    local doy=$(($mjd - $mjd0))
    while [ $doy -le 0 ]; do
        year=$(($year - 1))
        mjd0=$(ymd2mjd $year 1 1)
        doy=$(($mjd - $mjd0 + 1))
    done
    printf "%d %03d\n" $year $doy
}

main "$@"   
