import matplotlib.pyplot as plt
import os
import copy
import math
from math import nan,isnan
import sys

class sitatom(object):
    def __init__(self):
        self.sitname_base=""
        self.sitname_rove=""

        self.satlist_GPS = list()
        self.satlist_GPS = ['G01','G02','G03','G04','G05','G06','G07','G08','G09','G10',
            'G11','G12','G13','G14','G15','G16','G17','G18','G19','G20','G21',
            'G22','G23','G24','G25','G26','G27','G28','G29','G30','G31','G32'
            ]
        self.satlist_BD2 = list()
        self.satlist_BD2 = ['C01','C02','C03','C04','C05','C06','C07','C08','C09','C10',
            'C11','C12','C13','C14','C15','C16'
            ]
        self.satlist_BD3 = list()
        self.satlist_BD3 = ['C19','C20','C21','C22','C23','C24','C25','C26','C27','C28',
            'C29','C30','C32','C33','C34','C35','C36','C37','C38','C39','C40',
            'C41','C42','C43','C44','C45','C46',
            ]
        self.satlist_GAL = list()
        self.satlist_GAL = ['E01','E02','E03','E04','E05','E06','E07','E08','E09','E10',
            'E11','E12','E13','E14','E15','E18','E19','E20','E21','E22','E23',
            'E24','E25','E26','E27','E28','E29','E30','E31','E32','E33','E34',
            'E35','E36'
            ]

        # 卫星列表
        self.satlist = self.satlist_GPS + self.satlist_BD2 + self.satlist_BD3 + self.satlist_GAL
        # 卫星个数
        self.nsat = len(self.satlist)
        # 系统个数
        self.nsys = 4
        self.csys = ["GPS","BDS-2","BDS-3","Galileo"]

        self.smjd=nan
        self.ssod=nan
        self.emjd=nan
        self.esod=nan
        self.smjdsod=nan
        self.emjdsod=nan
        self.intv = 0.00
        self.nepoch = 0

        # 1: ION, 2: ZTD
        self.atomtype=0
        # 测站单差值
        self.sdatomlist = []
        # 测站高度角
        self.sdelevlist = []
        # 双差参考星
        self.ddrsatlist = []
        # 测站双差值
        self.ddatomlist = []

    def initdata(self,mjd0,sod0,mjd1,sod1,intv,type):
        self.smjd=nan
        self.ssod=nan
        self.emjd=nan
        self.esod=nan
        self.smjdsod=nan
        self.emjdsod=nan
        self.intv = 0.00
        self.nepoch = 0
        self.atomtype = 0
        self.sdatomlist = []
        self.sdelevlist = []
        self.ddrsatlist = []
        self.ddatomlist = []
        t_nepoch = round(((mjd1-mjd0)*86400.0+(sod1-sod0))/intv)+1
        if t_nepoch > 0 and (type == 1 or type == 2):
            self.smjd=mjd0
            self.ssod=sod0
            self.emjd=mjd1
            self.esod=sod1
            self.smjdsod=mjd0+sod0/86400.0
            self.emjdsod=mjd1+sod1/86400.0
            self.intv = intv
            self.nepoch = t_nepoch
            self.atomtype=type
            self.sdatomlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.sdelevlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.ddrsatlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.ddatomlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]

def index_sys(cprn):
    csys = cprn[0:1]
    isys = nan
    satlist_BD2 = ['C01','C02','C03','C04','C05','C06','C07','C08','C09','C10',
        'C11','C12','C13','C14','C15','C16'
        ]
    satlist_BD3 = ['C19','C20','C21','C22','C23','C24','C25','C26','C27','C28',
        'C29','C30','C32','C33','C34','C35','C36','C37','C38','C39','C40',
        'C41','C42','C43','C44','C45','C46',
        ]

    if csys == "G":
        isys = 0
    elif csys == "C":
        if cprn in satlist_BD2:
            isys = 1
        elif cprn in satlist_BD3:
            isys = 2
    elif csys == "E":
        isys = 3
    return isys

def read_sdatomfile(intv, atomfile, sitatom, type=1):
    """
    type=1: extract from residual file, the value is [7]
                                                                   ION/ZTD
        ION CQ09 E36   59863   14400.00   55.010000  158.879900    1.233010
        0   1    2     3       4          5          6             7
    type=2: generate by bamboo, the value is [6]
                                                       ION/ZTD
        ION VR01 C26   59863   34170.00    0.000000    2.544381 CQ19
        0   1    2     3       4           5           6        7
    """
    if intv <= 0:
        return 0
    if not (type == 1 or type == 2):
        return 0
    mjd0=nan
    sod0=nan
    mjd1=nan
    sod1=nan
    # 1: ION, 2:ZTD
    atomtype=0
    try:
        # get the atom type, and initial the class
        fid = open(atomfile)
        for line in fid:
            if len(line.strip()) <= 0:
                continue
            value = line.split()

            # get the atom type
            if atomtype == 0:
                if value[0] == "ION":
                    atomtype=1
                elif value[0] == "ZTD":
                    atomtype=2
            
            # get the period of time
            mjd=float(value[3])
            sod=float(value[4])

            if isnan(mjd0) or isnan(sod0) or isnan(mjd1) or isnan(sod1):
                mjd0 = mjd
                sod0 = sod
                mjd1 = mjd
                sod1 = sod
            
            if (mjd0+sod0/86400.0) >= (mjd+sod/86400.0):
                mjd0 = mjd
                sod0 = sod
            if (mjd1+sod1/86400.0) <= (mjd+sod/86400.0):
                mjd1 = mjd
                sod1 = sod

        if atomtype == 0:
            return 0

        if isnan(mjd0) or isnan(sod0) or isnan(mjd1) or isnan(sod1):
            return 0

        nepoch = round(((mjd1-mjd0)*86400.0+(sod1-sod0))/intv)+1
        if nepoch <= 0:
            return 0
        
        sitatom.initdata(mjd0,sod0,mjd1,sod1,intv,atomtype)

        fid.close()
    except:
        return 0

    try:
        fid = open(atomfile)
        for line in fid:
            if len(line.strip()) <= 0:
                continue
            value = line.split()
            t_prn = value[2]
            t_mjd = float(value[3])
            t_sod = float(value[4])
            t_elev = float(value[5])
            t_atom = nan
            if type == 1:
                t_atom = float(value[7])
            elif type == 2:
                t_atom = float(value[6])
            t_mjdsod = t_mjd+t_sod/86400.0
            if t_mjdsod < sitatom.smjdsod or t_mjdsod > sitatom.emjdsod:
                continue
            if not t_prn.strip() in sitatom.satlist:
                continue
            isat = sitatom.satlist.index(t_prn.strip())
            iepoch = round((t_mjdsod - sitatom.smjdsod)*86400.0/sitatom.intv)
            if atomtype == 1 or atomtype == 2:
                sitatom.sdatomlist[isat][iepoch] = t_atom
                sitatom.sdelevlist[isat][iepoch] = t_elev

        fid.close()
    except:
        return 0

    return 1

def calc_ddatom(sitatom):
    if sitatom.nepoch <= 0:
        return 0
    
    # choose the reference satellite depended on elevation
    try:
        iref_m=[ nan for col in range(sitatom.nsys) ]
        for iepoch in range(sitatom.nepoch):
            iref=[ nan for col in range(sitatom.nsys) ]
            elev_max=[ nan for col in range(sitatom.nsys) ]
            # get the reference satellite in single epoch
            for isat in range(sitatom.nsat):
                isys = index_sys(sitatom.satlist[isat])
                if isnan(isys):
                    continue
                if isnan(sitatom.sdelevlist[isat][iepoch]):
                    continue
                if isnan(elev_max[isys]):
                    elev_max[isys] = sitatom.sdelevlist[isat][iepoch]
                    iref[isys]=isat
                if sitatom.sdelevlist[isat][iepoch]>=elev_max[isys]:
                    elev_max[isys] = sitatom.sdelevlist[isat][iepoch]
                    iref[isys]=isat

            for i in range(sitatom.nsys):
                if isnan(iref_m[i]):
                    iref_m[i]=iref[i]
                else:
                    if isnan(iref[i]):
                        continue
                    if iref_m[i] == iref[i]:
                        continue
                    # change iref when the elev of older ref < 30 degree or no data
                    if isnan(sitatom.sdelevlist[iref_m[i]][iepoch]):
                        iref_m[i]=iref[i]
                    elif sitatom.sdelevlist[iref_m[i]][iepoch]<30.0:
                        iref_m[i]=iref[i]

            # record the reference satellite
            for isat in range(sitatom.nsat):
                isys = index_sys(sitatom.satlist[isat])
                if isnan(isys):
                    continue
                if isnan(sitatom.sdelevlist[isat][iepoch]):
                    continue
                if isnan(iref_m[isys]):
                    continue
                sitatom.ddrsatlist[isat][iepoch]=iref_m[isys]
    except:
        return 0

    # calculate
    try:
        for iepoch in range(sitatom.nepoch):
            for isat in range(sitatom.nsat):
                if isnan(sitatom.sdatomlist[isat][iepoch]):
                    continue
                if isnan(sitatom.ddrsatlist[isat][iepoch]):
                    continue
                # miss the reference satellite.
                iref = sitatom.ddrsatlist[isat][iepoch]
                if iref == isat:
                    continue
                if isnan(sitatom.sdatomlist[iref][iepoch]):
                    continue
                sitatom.ddatomlist[isat][iepoch] = sitatom.sdatomlist[isat][iepoch] - sitatom.sdatomlist[iref][iepoch]
    except:
        return 0

    return 1

def plot_ddatom(sitatom):
    if sitatom.nepoch <= 0:
        return 0
    try:
        for isys in range(sitatom.nsys):
            if isys == 0:
                ax1 = plt.subplot(2,2,isys+1)
            else:
                ax2 = plt.subplot(2,2,isys+1,sharey=ax1)
            plt.xlabel("Time/epoch")
            plt.ylabel("Value/m")
            plt.title(sitatom.csys[isys])
            for isat in range(sitatom.nsat):
                i = index_sys(sitatom.satlist[isat])
                if isnan(i):
                    continue
                if not isys == i:
                    continue
                x_time = [ i for i in range(sitatom.nepoch)]
                plt.plot(x_time, sitatom.ddatomlist[isat])
        #plt.set_size_inches(1.2, 2.8)
        plt.tight_layout()
        plt.savefig('plot_atom.png',dpi=600,bbox_inches='tight')
        #plt.show()
        i=0
    except:
        return 0
    return 1

def main(intv,type,atomfile):
    # check input
    try:
        intv=float(intv)
    except:
        print("***ERROR(py_drawDDAtom.py): the input interval maybe wrong: ",intv)
        return
    if intv <= 0:
        print("***ERROR(py_drawDDAtom.py): the input interval maybe wrong: ",intv)
        return

    try:
        type=int(type)
    except:
        print("***ERROR(py_drawDDAtom.py): the input type of atomfile format maybe wrong: ",type)
        return
    if not (type == 1 or type ==2):
        print("***ERROR(py_drawDDAtom.py): the input type of atomfile format maybe wrong: ",type)
        return

    if not os.path.exists(atomfile):
        print("***ERROR(py_drawDDAtom.py): Not find %s" %(atomfile))
        return

    # establish class
    t_sit = sitatom()

    if not read_sdatomfile(intv,atomfile,t_sit,type):
        return

    if not calc_ddatom(t_sit):
        return

    if not plot_ddatom(t_sit):
        return

if __name__  == '__main__':
    message='Usage: drawDDAtom.py intv type ion/ztdfile'
    if not len(sys.argv)==4:
        print(message)
        sys.exit()

    main(sys.argv[1],sys.argv[2],sys.argv[3])
    